import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import java.util.*;
import java.lang.*;
import java.lang.Thread.*;

//      Philosopher
//      Active class that extends Thread and sets up the framework for the Dining Philosophers Problem
public class Philosopher extends Thread {
    private static final Color THINK_BLACK = Color.black;
    private static final Color WAIT_RED = Color.red;
    private static final Color EAT_GREEN = Color.green;
    private static final double THINK_TIME = 4.0;
    private static final double FUMBLE_TIME = 5.0;
        // time between becoming hungry and grabbing first fork
    private static final double EAT_TIME = 3.0;

    private Server c;
    private Table t;
    private static final int XSIZE = 50;
    private static final int YSIZE = 50;
    private int x;
    private int y;
    private Fork left_fork;
    private Fork right_fork;
    private Random prn;
    private Color color;

    // Constructor.
    // cx and cy indicate coordinates of center
    public Philosopher(Table T, int cx, int cy,
                       Fork lf, Fork rf, Server C) {
        t = T;
        x = cx;
        y = cy;
        left_fork = lf;
        right_fork = rf;
        c = C;
        prn = new Random();
        color = THINK_BLACK;
    }

    // start method of Thread calls run
    //
    public void run() {
        for (;;) {
            try {
                if (c.gate()) delay(EAT_TIME/2.0);
                think();
                if (c.gate()) delay(THINK_TIME/2.0);
                hunger();
                if (c.gate()) delay(FUMBLE_TIME/2.0);
                eat();
            } catch(ResetException e) { 
                color = THINK_BLACK;
                t.repaint();
            }
        }
    }

    // render self
    //
    public void draw(Graphics g) {
        g.setColor(color);
         g.fillRect(x-XSIZE/2, y-YSIZE/2, XSIZE, YSIZE);
        
    }

    // sleep for secs +- FORK (%) seconds
    //
    private static final double FORK = 0.2;
    private void delay(double secs) throws ResetException {
        double ms = 1000 * secs;
        int window = (int) (2.0 * ms * FORK);
        int add_in = prn.nextInt() % window;
        int original_duration = (int) ((1.0-FORK) * ms + add_in);
        int duration = original_duration;
        for (;;) {
            try {
                Thread.sleep(duration);
                return;
            } catch(InterruptedException e) {
                if (c.isReset()) {
                    throw new ResetException();
                } else {        // suspended
                    c.gate();   // wait until resumed
                    duration = original_duration / 2;
                    // don't wake up instantly; sleep for about half
                    // as long as originally instructed
                }
            }
        }
    }

    private void think() throws ResetException {
        color = THINK_BLACK;
        t.repaint();
        delay(THINK_TIME);
    }

    private void hunger() throws ResetException {
        color = WAIT_RED;
        t.repaint();
        delay(FUMBLE_TIME);
        left_fork.acquire(x, y);
        yield();    // you aren't allowed to remove this
        right_fork.acquire(x, y);
    }

    private void eat() throws ResetException {
        color = EAT_GREEN;
        t.repaint();
        delay(EAT_TIME);
        left_fork.release();
        yield();    // you aren't allowed to remove this
        right_fork.release();
    }
}
